import redis
import re
import uuid
import mysql.connector
import time
import argparse
import threading
import queue

# === 参数配置 ===
parser = argparse.ArgumentParser(description="XrayR 多线程设备检测器")
parser.add_argument("--interval", type=int, default=10, help="每次检查的时间间隔（秒）")
parser.add_argument("--max-devices", type=int, default=4, help="最大设备数，超出替换 UUID")
parser.add_argument("--threads", type=int, default=8, help="处理线程数")
args = parser.parse_args()

# === Redis 和 MySQL 配置 ===
r = redis.Redis(host='16.42.11.11', port=6379, password='123456', db=0)
db_config = {
    "host": "16.42.11.11",
    "user": "xb",
    "password": "nLMMX3111FySH4",
    "database": "xb",
    "port": 3306
}

# === 数据结构 ===
key_queue = queue.Queue()
user_device_map = {}
lock = threading.Lock()


# === 提取 IP 函数（兼容 \xad 和 \xae） ===
def extract_ip_from_raw(raw):
    try:
        if b'\xad' in raw:
            ip_raw = raw.split(b'\xad')[1].rstrip(b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09')
            ip = ip_raw.decode(errors='ignore')
            if re.match(r'\d+\.\d+\.\d+\.\d+', ip):
                return ip
        elif b'\xae' in raw:
            parts = raw.split(b'\xae')[1:]
            for part in parts:
                ip_raw = part.split(b'\x01')[0]
                ip = ip_raw.decode(errors='ignore')
                if re.match(r'\d+\.\d+\.\d+\.\d+', ip):
                    return ip
    except:
        return None
    return None


# === Key 处理线程 ===
def process_keys():
    while True:
        try:
            key = key_queue.get(timeout=5)
        except queue.Empty:
            break
        try:
            key_str = key.decode(errors='ignore')
            match = re.search(r'\d+\|(.+?)@v2board\.user\|(\d+)', key_str)
            if not match:
                continue

            uuid_str = match.group(1)
            user_id = match.group(2)

            raw = r.get(key)
            if not raw:
                continue

            ip = extract_ip_from_raw(raw)
            if not ip:
                continue

            with lock:
                if uuid_str not in user_device_map:
                    user_device_map[uuid_str] = {
                        "user_id": user_id,
                        "ips": set()
                    }
                user_device_map[uuid_str]["ips"].add(ip)

        except Exception as e:
            print(f"❌ 处理 key 出错：{e}")
        finally:
            key_queue.task_done()


# === 主扫描逻辑 ===
def scan_redis_keys(pattern="*@v2board.user|*"):
    cursor = 0
    print("🔍 开始 SCAN Redis Key")
    while True:
        cursor, keys = r.scan(cursor=cursor, match=pattern, count=1000)
        for k in keys:
            key_queue.put(k)
        if cursor == 0:
            break
    print(f"✅ SCAN 完成，总计 key 数：{key_queue.qsize()}")


# === 主循环 ===
while True:
    start = time.time()
    print(f"\n==== ⏱️ 开始检查：{time.strftime('%Y-%m-%d %H:%M:%S')} ====")
    user_device_map.clear()

    try:
        scan_redis_keys()

        # 启动处理线程
        threads = []
        for _ in range(args.threads):
            t = threading.Thread(target=process_keys)
            t.start()
            threads.append(t)

        # 等待所有线程处理完成
        for t in threads:
            t.join()

        print(f"📊 已聚合用户数：{len(user_device_map)}")

        # === 替换 UUID ===
        db = mysql.connector.connect(**db_config)
        cursor = db.cursor()

        for uuid_str, data in user_device_map.items():
            user_id = data["user_id"]
            ip_list = list(data["ips"])
            device_count = len(ip_list)

            print(f"\n🧑 用户 ID: {user_id} / UUID: {uuid_str}")
            print(f"  🌐 设备数：{device_count}")
            for ip in ip_list:
                print(f"   - {ip}")

            if device_count > args.max_devices:
                new_uuid = str(uuid.uuid4())
                try:
                    sql = "UPDATE v2_user SET uuid = %s WHERE uuid = %s"
                    cursor.execute(sql, (new_uuid, uuid_str))
                    db.commit()
                    print(f"  ⚠️ 超过限制，UUID 已替换为：{new_uuid}")
                except Exception as e:
                    print(f"  ❌ UUID 替换失败：{e}")

        cursor.close()
        db.close()

    except Exception as e:
        print(f"🔥 执行失败：{e}")

    duration = round(time.time() - start, 2)
    print(f"\n✅ 本轮检查完成，耗时：{duration} 秒")
    time.sleep(args.interval)